function POD = computePODBasis(prob,Xi,Norm,tol)

  tol = eval('tol','1e-10');
  
  POD.Xi = Xi;
  POD.N = size(Xi,2);
  POD.Norm = Norm;

% compute snapshots
  U = zeros(prob.fem.dof.num,POD.N);
  for n=1:size(U,2)
    U(:,n) = getSol(prob,Xi(:,n));
  end

% compute correlation
  C = U'*prob.fem.innProd.(POD.Norm)*U;
  
% compute eigenvalues and eigenvectors
  [V,Lambda] = eig(C);
  Lambda = diag(Lambda);

% determine d  
  d = find(~(Lambda>tol),1,'first')-1;

  if ~isempty(d)
  % shrink eigenvalues and eigenvectors 
    V = V(:,1:d);
    Lambda = Lambda(1:d);
  else
    d = length(Lambda);
  end
  
  POD.d = d;
  
% compute POD basis
  POD.Basis = U*(V*diag(Lambda.^-.5));
  POD.Lambda = Lambda;
  POD.traceC = trace(C);

% compute pre assembled-rhs
  POD.pre.rhs.theta = prob.fem.pre.rhs.theta;
  POD.pre.rhs.L = POD.Basis'*prob.fem.pre.rhs.L;
  
% compute pre assembled-lhs
  Q = length(prob.fem.pre.lhs);
  POD.pre.lhs = struct('theta',{},'K',{});

  for q=1:Q
    K = POD.Basis'*prob.fem.pre.lhs(q).K*POD.Basis;
    POD.pre.lhs(q) = struct('theta', prob.fem.pre.lhs(q).theta, 'K', K);
  end
